@@ -8,6 +8,7 @@ ChromeUtils.import("resource://gre/modules/AppConstants.jsm");
 
 XPCOMUtils.defineLazyModuleGetters(this, {
   WindowsGPOParser: "resource:///modules/policies/WindowsGPOParser.jsm",
+  macOSPoliciesParser: "resource:///modules/policies/macOSPoliciesParser.jsm",
   Policies: "resource:///modules/policies/Policies.jsm",
   PoliciesValidator: "resource:///modules/policies/PoliciesValidator.jsm",
 });
@@ -94,16 +95,19 @@ EnterprisePoliciesManager.prototype = {
   },
 
   _chooseProvider() {
+    let provider = null;
     if (AppConstants.platform == "win") {
-      let gpoProvider = new GPOPoliciesProvider();
-      if (gpoProvider.hasPolicies) {
-        return gpoProvider;
-      }
+      provider = new WindowsGPOPoliciesProvider();
+    } else if (AppConstants.platform == "macosx") {
+      provider = new macOSPoliciesProvider();
+    }
+    if (provider && provider.hasPolicies) {
+      return provider;
     }
 
-    let jsonProvider = new JSONPoliciesProvider();
-    if (jsonProvider.hasPolicies) {
-      return jsonProvider;
+    provider = new JSONPoliciesProvider();
+    if (provider.hasPolicies) {
+      return provider;
     }
 
     return null;
@@ -151,7 +155,7 @@ EnterprisePoliciesManager.prototype = {
   },
 
   _callbacks: {
-    // The earlist that a policy callback can run. This will
+    // The earliest that a policy callback can run. This will
     // happen right after the Policy Engine itself has started,
     // and before the Add-ons Manager has started.
     onBeforeAddons: [],
@@ -418,7 +422,7 @@ class JSONPoliciesProvider {
   }
 }
 
-class GPOPoliciesProvider {
+class WindowsGPOPoliciesProvider {
   constructor() {
     this._policies = null;
 
@@ -452,5 +456,29 @@ class GPOPoliciesProvider {
   }
 }
 
+class macOSPoliciesProvider {
+  constructor() {
+    this._policies = null;
+    let prefReader = Cc["@mozilla.org/mac-preferences-reader;1"]
+                       .createInstance(Ci.nsIMacPreferencesReader);
+    if (!prefReader.policiesEnabled()) {
+      return;
+    }
+    this._policies = macOSPoliciesParser.readPolicies(prefReader);
+  }
+
+  get hasPolicies() {
+    return this._policies !== null;
+  }
+
+  get policies() {
+    return this._policies;
+  }
+
+  get failed() {
+    return this._failed;
+  }
+}
+
 var components = [EnterprisePoliciesManager];
 this.NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
